/**************************************************************************
*
* Filename:    TC08ConsoleExample.cs
*
* Copyright:   Pico Technology Limited 2011
*
* Author:      CPY
*
* Description:
*   This is a console-mode program that demonstrates how to use the
*   TC08 driver using .NET
*
* Examples:
*  
*
* History:
*     23/05/2011	CPY	Created
*
* Revision Info: "file %n date %f revision %v"
*						""
*
***************************************************************************/

using System;
using System.IO;
using System.Threading;

namespace TC08example
{
   

    class ConsoleExample
    {
        private readonly short _handle;
        public const int USBTC08_MAX_CHANNELS = 8;
        public const char TC_TYPE_K = 'K';
        public const int PICO_OK = 1;
       

    private static void WaitForKey()
    {
        while (!Console.KeyAvailable) Thread.Sleep(100);
        if (Console.KeyAvailable) Console.ReadKey(true); // clear the key
    }

    /****************************************************************************
     * Read the device information
     ****************************************************************************/
   void GetDeviceInfo()
    {
        System.Text.StringBuilder line = new System.Text.StringBuilder(256);

        if (_handle >= 0)
        {
            Imports.TC08GetFormattedInfo(_handle, line, 256);
            Console.WriteLine("{0}", line);
        }
    }

    /****************************************************************************
     * Read temperature information from the unit
     ****************************************************************************/
   unsafe void GetValues()
   {
       short status;
       short chan;
       float[] tempbuffer = new float[9]; 
       short overflow;
       int lines=0;


       Console.Write("\n");
       // label the columns
       for (chan = 1; chan <= USBTC08_MAX_CHANNELS; chan++)
       {
           Console.Write("Chan{0}:    ", chan);
       }
       Console.Write("\n");

       do
       { 
           status = Imports.TC08GetSingle(_handle, tempbuffer, &overflow, Imports.TempUnit.USBTC08_UNITS_CENTIGRADE);
           if (status == PICO_OK)
           {
                for (chan = 1; chan <= USBTC08_MAX_CHANNELS; chan++)
                {
                    Console.Write("{0:0.0000}   ", tempbuffer[chan]);
                }
                Console.Write("\n");
                Thread.Sleep(1000);
           }

           if (++lines > 9)
           {
               Console.WriteLine("Hit any key to stop....\n");
               Console.WriteLine("Cold Junction Temperature: {0:0.0000}   \n", tempbuffer[0]);

               lines = 0;
               for (chan = 1; chan <= USBTC08_MAX_CHANNELS; chan++)
               {
                   Console.Write("Chan{0}:    ", chan);
               }
               Console.Write("\n");
           }
       } while (Win32Interop._kbhit() == 0);
       char ch = (Console.ReadKey().KeyChar);       // use up keypress
       status = Imports.TC08Stop(_handle);
   }



    /****************************************************************************
    *  Set channels 
    ****************************************************************************/
    void SetChannels()
    {
	    short channel;
	    short ok;

	    for (channel = 0; channel <= USBTC08_MAX_CHANNELS; channel++)
	    {
            ok = Imports.TC08SetChannel(_handle, channel, TC_TYPE_K);

	    }
    }

    /****************************************************************************
    *  Run
    ****************************************************************************/
    public void Run()
    {
       
         //// main loop - read key and call routine
        char ch = ' ';
        while (ch != 'X')
        {
            Console.WriteLine("");
            Console.WriteLine("I - Device Info");
            Console.WriteLine("G - Get Temperatures");
            Console.WriteLine("X - exit");
            Console.WriteLine("Operation:");

            ch = char.ToUpper(Console.ReadKey().KeyChar);

            Console.WriteLine("\n");
            switch (ch)
            {
                case 'I':
                    GetDeviceInfo();
                    break;

                case 'G':
                    SetChannels();
                    GetValues();
                    break;

                case 'X':
                    /* Handled by outer loop */
                    break;

                default:
                    Console.WriteLine("Invalid operation");
                    break;
            }
        }
    }


    private ConsoleExample(short handle)
    {
        _handle = handle;
    }


    static void Main()
    {
      Console.WriteLine("TC08 driver example program");
      Console.WriteLine("Version 1.0\n");

    

      //open unit and show splash screen
      Console.WriteLine("\n\nOpening the device...");

      short handle = Imports.TC08OpenUnit();
      Console.WriteLine("Handle: {0}", handle);
      if (handle == 0)
      {
        Console.WriteLine("Unable to open device");
        Console.WriteLine("Error code : {0}", handle);
        WaitForKey();
      }
      else
      {
        Console.WriteLine("Device opened successfully\n");

        ConsoleExample consoleExample = new ConsoleExample(handle);
        consoleExample.Run();

        Imports.TC08CloseUnit(handle);
      }
    }
  }
}  
